<?php

namespace Litecms\Maintenance\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Maintenance\Forms\Proposal as ProposalForm;
use Litecms\Maintenance\Http\Requests\ProposalResourceRequest;
use Litecms\Maintenance\Http\Resources\ProposalResource;
use Litecms\Maintenance\Http\Resources\ProposalsCollection;
use Litecms\Maintenance\Models\Proposal;
use Litecms\Maintenance\Actions\ProposalAction;
use Litecms\Maintenance\Actions\ProposalActions;


/**
 * Resource controller class for proposal.
 */
class ProposalResourceController extends BaseController
{

    /**
     * Initialize proposal resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ProposalForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.maintenance.modules'), 'maintenance', guard_url('maintenance'));
            return $next($request);
        });
    }

    /**
     * Display a list of proposal.
     *
     * @return Response
     */
    public function index(ProposalResourceRequest $request)
    {
        $request = $request->all();
        $page = ProposalActions::run('paginate', $request);

        $data = new ProposalsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('maintenance::proposal.names'))
            ->view('maintenance::proposal.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display proposal.
     *
     * @param Request $request
     * @param Model   $proposal
     *
     * @return Response
     */
    public function show(ProposalResourceRequest $request, Proposal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProposalResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('maintenance::proposal.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('maintenance::proposal.show')
            ->output();
    }

    /**
     * Show the form for creating a new proposal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ProposalResourceRequest $request, Proposal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProposalResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('maintenance::proposal.name'))
            ->view('maintenance::proposal.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new proposal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProposalResourceRequest $request, Proposal $model)
    {
        try {
            $request = $request->all();
            $model = ProposalAction::run('store', $model, $request);
            $data = new ProposalResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('maintenance::proposal.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('maintenance/proposal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/maintenance/proposal'))
                ->redirect();
        }

    }

    /**
     * Show proposal for editing.
     *
     * @param Request $request
     * @param Model   $proposal
     *
     * @return Response
     */
    public function edit(ProposalResourceRequest $request, Proposal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProposalResource($model);
        // return view('maintenance::proposal.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('maintenance::proposal.name'))
            ->view('maintenance::proposal.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the proposal.
     *
     * @param Request $request
     * @param Model   $proposal
     *
     * @return Response
     */
    public function update(ProposalResourceRequest $request, Proposal $model)
    {
        try {
            $request = $request->all();
            $model = ProposalAction::run('update', $model, $request);
            $data = new ProposalResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('maintenance::proposal.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('maintenance/proposal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('maintenance/proposal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the proposal.
     *
     * @param Model   $proposal
     *
     * @return Response
     */
    public function destroy(ProposalResourceRequest $request, Proposal $model)
    {
        try {

            $request = $request->all();
            $model = ProposalAction::run('destroy', $model, $request);
            $data = new ProposalResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('maintenance::proposal.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('maintenance/proposal/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('maintenance/proposal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
