<?php

namespace Litecms\Maintenance\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Maintenance\Forms\Workorder as WorkorderForm;
use Litecms\Maintenance\Http\Requests\WorkorderResourceRequest;
use Litecms\Maintenance\Http\Resources\WorkorderResource;
use Litecms\Maintenance\Http\Resources\WorkordersCollection;
use Litecms\Maintenance\Models\Workorder;
use Litecms\Maintenance\Actions\WorkorderAction;
use Litecms\Maintenance\Actions\WorkorderActions;


/**
 * Resource controller class for workorder.
 */
class WorkorderResourceController extends BaseController
{

    /**
     * Initialize workorder resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = WorkorderForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.maintenance.modules'), 'maintenance', guard_url('maintenance'));
            return $next($request);
        });
    }

    /**
     * Display a list of workorder.
     *
     * @return Response
     */
    public function index(WorkorderResourceRequest $request)
    {
        $request = $request->all();
        $page = WorkorderActions::run('paginate', $request);

        $data = new WorkordersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('maintenance::workorder.names'))
            ->view('maintenance::workorder.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display workorder.
     *
     * @param Request $request
     * @param Model   $workorder
     *
     * @return Response
     */
    public function show(WorkorderResourceRequest $request, Workorder $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WorkorderResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('maintenance::workorder.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('maintenance::workorder.show')
            ->output();
    }

    /**
     * Show the form for creating a new workorder.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(WorkorderResourceRequest $request, Workorder $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WorkorderResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('maintenance::workorder.name'))
            ->view('maintenance::workorder.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new workorder.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(WorkorderResourceRequest $request, Workorder $model)
    {
        try {
            $request = $request->all();
            $model = WorkorderAction::run('store', $model, $request);
            $data = new WorkorderResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('maintenance::workorder.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('maintenance/workorder/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/maintenance/workorder'))
                ->redirect();
        }

    }

    /**
     * Show workorder for editing.
     *
     * @param Request $request
     * @param Model   $workorder
     *
     * @return Response
     */
    public function edit(WorkorderResourceRequest $request, Workorder $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new WorkorderResource($model);
        // return view('maintenance::workorder.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('maintenance::workorder.name'))
            ->view('maintenance::workorder.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the workorder.
     *
     * @param Request $request
     * @param Model   $workorder
     *
     * @return Response
     */
    public function update(WorkorderResourceRequest $request, Workorder $model)
    {
        try {
            $request = $request->all();
            $model = WorkorderAction::run('update', $model, $request);
            $data = new WorkorderResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('maintenance::workorder.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('maintenance/workorder/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('maintenance/workorder/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the workorder.
     *
     * @param Model   $workorder
     *
     * @return Response
     */
    public function destroy(WorkorderResourceRequest $request, Workorder $model)
    {
        try {

            $request = $request->all();
            $model = WorkorderAction::run('destroy', $model, $request);
            $data = new WorkorderResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('maintenance::workorder.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('maintenance/workorder/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('maintenance/workorder/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
