<?php

namespace Litecms\Maintenance\Http\Controllers;

use Exception;
use Litecms\Maintenance\Actions\WorkorderWorkflow;
use Litecms\Maintenance\Http\Requests\WorkorderWorkflowRequest;
use Litecms\Maintenance\Http\Resources\WorkorderResource;
use Litecms\Maintenance\Models\Workorder;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class WorkorderWorkflowController extends BaseController
{
    /**
     * Action controller function for workorder.
     *
     * @param Model $workorder
     * @param action next action for the workorder.
     *
     * @return Response
     */
    public function __invoke(WorkorderWorkflowRequest $request, Workorder $workorder, $trasition)
    {
        try {
            $request = $request->all();
            $workorder = WorkorderWorkflow::run($trasition, $workorder, $request);
            $data = new WorkorderResource($workorder);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('maintenance::workorder.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('maintenance/workorder/' . $workorder->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('maintenance/workorder/' . $workorder->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the workorder and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $workorder
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $workorder, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $workorder = Workorder::findBySignedId($workorder);
        $transitions = $this->transitions($workorder, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('maintenance::workorder.name') . ' ' . trans('Approvals'))
            ->view('maintenance::public.workorder.workflow')
            ->layout('mobile')
            ->data(compact('workorder', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the workorder.
     *
     * @param PublicRequest $request
     * @param string $workorder
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $workorder, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $workorder = Workorder::findBySignedId($workorder);
        $user = user();
        $workorder = WorkorderWorkflow::run($transition, $workorder, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($workorder, $user)
    {
        $transitions = [];
        foreach ($workorder->workflow()->transitions($workorder) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.workorder.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'workorder' => $workorder->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $workorder->workflow()->form($value);
            $array['label'] = trans('maintenance::workorder.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
